#ifndef _COURS_EIGENVALES_BY_BISSECTION_H
#define _COURS_EIGENVALES_BY_BISSECTION_H
//
// compute by bissection the eigenvales of
//
// references:
// P. Ciarlet, 
// Introduction a` l'analyse numerique matricielle et a` l'optimisation,
// Masson, 1982
// page 122
//
//	    [   0   a(0)   	...........	]
//	    [  a(0)   0	  a(1) 	...........	]
//  A_N  =  [					]
//          [					]
//	    [  ........    	  0    a(N-2)	]
//	    [........  		a(N-2)   0	]
//
//
//	a      (0:N-2)	: input coefficients
//	lambda (0:N-1)  : output eigenvalues = roots of P_N(x)
//
//	P_0 = 1
//	P_1 = -x
//	P_i = -x*P_{i-1} - a(i-2)^2*P_{i-2},	i=2..N-1
//
// computing time: 
//
//	T(N) = O(ndgits*N)
//
// Performances for Legendre-Gauss-Lobatto for N <= 40 with T=double
// See legendre_gauss_lobatto_bissection_tst.cc
//
#include <math.h>
#include <vector>
#include "numeric_constant.h"

template <typename Size, typename Container>
typename Container::value_type
eigenvalue_by_bissection (const Container& a, Size i)
{
	typedef typename Container::value_type T;
	if (a.size() == 0) return 0;
	if (a.size() == 1) return (i == 0) ? -a[0] : a[0]; 

	Size N = a.size() + 1;

	const T    epsilon = numeric_constant<T>::epsilon();

	T a_norm1 = 0; // norm1(A) = max_j sum_i |aij|
	for (Size j = 0; j < a.size()-1; j++)
	    a_norm1 = std::max (a_norm1, fabs(a[j+1])+fabs(a[j]));
	
	std::vector<T> p (N+1);
	const int positive = 1;
	const int negative = -1;
	std::vector<int> sign_p (N+1);

	T lambda_sup =  (1+sqrt(epsilon))*a_norm1;
	T lambda_inf = - lambda_sup;
	T lambda =      (lambda_sup + lambda_inf)/2;
	T delta  = fabs (lambda_sup - lambda_inf);
	const Size k_max   =  int(1 - log2(epsilon/delta));
	for (Size k = 0; k < k_max && delta >= epsilon; k++) {

	    lambda = (lambda_sup + lambda_inf)/2;

	    p[0] = 1;
	    p[1] = -lambda;
	    for (Size j = 2; j < N+1; j++) {
	        p[j] = - lambda*p[j-1] - a[j-2]*a[j-2]*p[j-2];
	    }
	    sign_p [0] = positive;
	    for (Size j = 1; j < N+1; j++) {
		T pval = (fabs(p[j]) <= epsilon) ? p[j-1] : p[j];
		sign_p [j] = (pval > 0) ? positive : negative;
	    }
	    Size count_change = 0;
	    for (Size j = 1; j < N+1; j++) {
		if (sign_p[j] != sign_p[j-1]) count_change++;
	    }
	    if (count_change >= i+1) { // lambda_{i,N} is in [lambda_inf, lambda[
		lambda_sup = lambda;
	    } else { 		     // lambda_{i,N} is in [lambda, lambda_sup[
		lambda_inf = lambda;
	    }
	    lambda =      (lambda_sup + lambda_inf)/2;
	    delta  = fabs (lambda_sup - lambda_inf);
	}
	return lambda;
}
#endif // _COURS_EIGENVALES_BY_BISSECTION_H
