#ifndef _COURS_LEGENDRE_GAUSS_H
#define _COURS_LEGENDRE_GAUSS_H
//
// compute the Gauss collocation points and weights 
// for Legendre polynomial at order N for Omega = ]-1,1[ 
//
// the formulae is exact in P_{2N+1}
//
//  x [0:N] : the colocation points in increasing order
//  w [0:N] : the associated weights
//
// reference : C. Bernardi and Y. Maday
// Spectral methods
// Handbook for Numerical analysis, vol. 5,
// Elsevier, 1997
// pages 244-249
//
// Performances: for N=40, T=double, the sum_i w_i = 2.00002 . 2
// compare with octave:
//
//		n = [1:N-1];           
//		d = n./sqrt(4*n.^2-1);  
//		J = diag(d,1)+diag(d,-1);
//		r = sort(eig(sparse(J)));
//
// The weights could be computed directly with the norm of the eigenvector:
// see Bernardi-Maday, p. 244
//
#include "legendre.h"
#include "tqli_wrap.h"
#include <vector>
#include <cmath>
#include <algorithm>

template <typename Container>
void legendre_gauss (Container& x, Container& w)
{
        typedef typename Container::value_type T;
	if (x.size() == 0) { return; }
	size_t N = x.size() - 1;
	if (N == 0) { x [0] = 0; w[0] = 2; return; }
	Container alpha (N);
	for (size_t i = 0; i <= N-1; i++)
	    alpha[i] = (i+1)/std::sqrt(T(4*sqr(i+1)-1));

	// roots of L(N)
        fill (x.begin(), x.end(), T(0));
        tqli_wrap (x.begin(), alpha.begin(), N+1);
        std::sort (x.begin(), x.end());
	if (N % 2 == 0) x[N/2] = 0;

	// w_i = 2/((1-x^2)*(L'(N+1)(x))^2)
	legendre_derivative<T> dL (N+1);
	for (size_t i = 0; i <= N/2; i++)
            w[i] = w[N-i] = 2/((1 - sqr (x[i]))*sqr (dL (x[i])));
}
#endif // _COURS_LEGENDRE_GAUSS_H
